#define NOTE_A4  440
#define NOTE_C4  262

#define numRows 4
#define numCols 9
#define maxRowNum 7

int bottomItemX;
byte bottomItemWidth;
int bottomItemOldX;
int ballX;
int ballY;
int itemDirX;
int itemDirY;
byte loopCount=false;
unsigned int blocks[8];
byte blockStore[gameMaxX][maxRowNum];

void breakout_loop()
{
    int ret = nunchuck_get_data();
  if (!ret) {
    delay(500);
    nunchuck_init();
  } else {
    int joyX=nunchuck_joyx();
    if (joyX<64) bottomItemX--;
    if (joyX>172) bottomItemX++;
    bottomItemX=constrain(bottomItemX,gameMinX-bottomItemWidth+1,gameMaxX);

    if (loopCount) {
      breakout_move_ball();
    }
    loopCount=!loopCount;
    breakout_draw_paddles();
    delay(speed);
  }
}

void breakout_setup_game()
{
  Serial.println("Breakout");
  dmd.clearScreen(0);
  add_wall(gameMinX,gameMinY,gameMinX,gameMaxY);
  add_wall(gameMaxX,gameMinY,gameMaxX,gameMaxY);
  add_wall(gameMinX,gameMinY,gameMaxX,gameMinY);

  for (byte x=0;x<gameMaxX;x++) {
    for (byte y=0;y<maxRowNum;y++) {
      blockStore[x][y-1]=0;
    }
  }
  for (byte i=0;i<15;i++) blocks[i]=0;
  byte blockNum=0;
  for (byte y=0;y<numRows;y++) {
    for (byte x=0;x<numCols;x++) {
      blockNum++;
      for (byte j=(x*3);j<=((x*3)+2);j++) {
        for (byte k=(y*2);k<=((y*2)+1);k++) {
          dmd.writePixel(j+2,k+1,7);
          blockStore[j+2][k]=blockNum;
        }
      }
      bitWrite(blocks[blockNum/16],blockNum % 16,1);
    }
  }

  set_paddle();
  speed=10;
  points=5; // Number of balls
  update_scorebar(scoreY);
  nunchuck_wait();
}

void set_paddle()
{
  bottomItemX=gameCentreX-2;
  bottomItemWidth=5;
  bottomItemOldX=0;
  ballX=gameCentreX+(random(3)-2);
  ballY=gameMaxY-1;
  itemDirX=0;
  itemDirY=-1;
  breakout_draw_paddles();
}

void breakout_draw_paddles()
{
  if (bottomItemOldX != bottomItemX) {
    for (byte i=gameMinX;i<=gameMaxX;i++) {
      if ((i >= bottomItemX) && (i<(bottomItemX+bottomItemWidth))) {
        dmd.writePixel(i,gameMaxY,7);
      } 
      else {
        dmd.writePixel(i,gameMaxY,0);
      }
    }
    bottomItemOldX=bottomItemX;
  }
}


void breakout_move_ball()
{
  byte oldBallX = ballX;
  byte oldBallY = ballY;
  ballX=ballX+itemDirX;
  ballY=ballY+itemDirY;
  if ((ballX <= (gameMinX)) || (ballX >= (gameMaxX))) {
    itemDirX=-itemDirX;
    ballX=ballX+itemDirX;
  }

  int nextX = ballX+itemDirX;
  int nextY = ballY+itemDirY;
  if (nextY <= gameMinY) {
    itemDirY=-itemDirY;
    ballY=gameMinY+1;
    tone(speakerPin, NOTE_A4,200);
  } 
  else if (nextY >= gameMaxY) {
    if ((ballX < bottomItemX) || (ballX>=(bottomItemX+bottomItemWidth))) {
      // Missed it!
      dmd.writePixel(ballX,ballY,0);
      dmd.writePixel(oldBallX,oldBallY,0);
      points--;
      update_scorebar(scoreY);
      set_paddle();
      if (points==0) {
        display_gameover();
        display_score();
        breakout_setup_game();
      }
    } 
    else {
      if (ballX<(bottomItemX+(bottomItemWidth/2))-1) {
        itemDirX--;
      } 
      else if (ballX>(bottomItemX+(bottomItemWidth/2))+1) {
        itemDirX++;
      }
      tone(speakerPin, NOTE_C4,200);
      itemDirX=constrain(itemDirX,-1,1);
      itemDirY=-itemDirY;
    }
  } 
  else if ((nextX == gameMinX) || (nextX == gameMaxX)) {
    itemDirX=-itemDirX;
    tone(speakerPin, NOTE_A4,200);
  }
  else if (ballY <= maxRowNum) {
    byte blockNum = 0;
    if (blockStore[nextX][nextY-1] != 0) {
      blockNum = blockStore[nextX][nextY-1];
      itemDirX=-itemDirX;
      itemDirY=-itemDirY;
    } 
    else if (blockStore[nextX][ballY-1] != 0) {
      blockNum = blockStore[nextX][ballY-1];
      itemDirX=-itemDirX;
    } 
    else if (blockStore[ballX][nextY-1] != 0) {
      blockNum = blockStore[ballX][nextY-1];
      itemDirY=-itemDirY;
    }
    if (blockNum != 0) {
      tone(speakerPin, NOTE_A4+(blockNum*5),200);
      bitWrite(blocks[blockNum/16],blockNum % 16,0);
      for (byte x=0;x<gameMaxX;x++) {
        for (byte y=0;y<maxRowNum;y++) {
          if (blockStore[x][y-1] == blockNum) {
            dmd.writePixel(x,y,0);
            blockStore[x][y-1]=0;
          }
        }
      }
      int total=0;
      for (byte i=0;i<15;i++) total += blocks[i];
      if (total == 0) {
        level++;
        display_score();
        breakout_setup_game();  
        return;
      } 
    }
  }
  dmd.writePixel(ballX,ballY,WHITE);
  if ((ballX > gameMinX) && (ballX < gameMaxX) && (ballY > gameMinY)) {
    dmd.writePixel(oldBallX,oldBallY,0);
  } 
}
