// Define special pong properties
int topPaddle;
int topPaddleOld;
byte topItemWidth;
byte aiPoints=0;

void pong_draw_paddles()
{
  if (topPaddleOld != topPaddle) {
    for (byte i=gameMinX;i<=gameMaxX;i++) {
      if ((i >= topPaddle) && (i<(topPaddle+topItemWidth))) {
        dmd.writePixel(i,gameMinY,7);
      } else {
        dmd.writePixel(i,gameMinY,0);
      }
    }
    topPaddleOld=topPaddle;
  }
  if (bottomItemOldX != bottomItemX) {
    for (byte i=gameMinX;i<=gameMaxX;i++) {
      if ((i >= bottomItemX) && (i<(bottomItemX+bottomItemWidth))) {
        dmd.writePixel(i,gameMaxY,7);
      } else {
        dmd.writePixel(i,gameMaxY,0);
      }
    }
    bottomItemOldX=bottomItemX;
  }
}

void pong_setup_board()
{
  dmd.clearScreen(0);
  for (byte i=gameMinX; i< gameMaxX; i++) {
    if (i&1) {
      dmd.writePixel(i,gameCentreY,3);
    } else {
       dmd.writePixel(i,gameCentreY-1,3);
    }
  }
  add_wall(gameMinX,gameMinY,gameMinX,gameMaxY);
  add_wall(gameMaxX,gameMinY,gameMaxX,gameMaxY);
}

void pong_loop()
{
  int ret = nunchuck_get_data();
  if (!ret) {
    delay(500);
    nunchuck_init();
  } else {
    int joyX=nunchuck_joyx();
    if (joyX<64) bottomItemX--;
    if (joyX>172) bottomItemX++;
    bottomItemX = constrain(bottomItemX,(gameMinX-bottomItemWidth+1),gameMaxX);
  
    pong_move_ai();
 
    if (loopCount) {
      pong_move_ball();
    }
    loopCount=!loopCount;
    pong_draw_paddles();
    delay(speed*3);
  }
}

void pong_move_ai()
{
  // Only move 2/3 of the time
  if (random(3) != 0) {
    if (ballX < (topPaddle+(topItemWidth/2))) {
      topPaddle--;
    } 
    else {
      topPaddle++;
    }
  }
}

void pong_setup_game()
{ 
  if (gameType==NONE) return;
  pong_setup_board();
  topPaddle=gameCentreX;
  bottomItemX=gameCentreX;
  topPaddleOld=0;
  bottomItemOldX=0;
  topItemWidth=5;
  bottomItemWidth=5;
  ballX=gameCentreX;
  ballY=gameCentreY;
  itemDirX=random(2);
  if (itemDirX==0) itemDirX=-1;
  itemDirY=random(2);
  if (itemDirY==0) itemDirY=-1;
  pong_draw_paddles();
  speed=10-points;
  speed=min(speed,9);
  nunchuck_wait();
}


void pong_move_ball()
{
  byte oldBallX = ballX;
  byte oldBallY = ballY;
  ballX=ballX+itemDirX;
  ballY=ballY+itemDirY;
  if ((ballX <= (gameMinX)) || (ballX >= (gameMaxX))) {
    itemDirX=-itemDirX;
    ballX=ballX+itemDirX;
    tone(speakerPin,NOTE_B4,200);
  }

  if (ballY <= (gameMinY+1)) {
    itemDirY=-itemDirY;
    if ((ballX < topPaddle) || (ballX>=(topPaddle+topItemWidth))) {
      // Missed it
      points++;
      dmd.writePixel(ballX,ballY,7);
      pong_display_score_update();
      pong_setup_game();
      update_scorebar(scoreY);
    } else {
      tone(speakerPin, NOTE_A4,200);
    }
  } else if (ballY >= (gameMaxY-1)) {
    itemDirY=-itemDirY;
    if ((ballX < bottomItemX) || (ballX>=(bottomItemX+bottomItemWidth))) {
      // Missed it!
      dmd.writePixel(ballX,ballY,7);
      aiPoints++;
      pong_display_score_update();
      pong_setup_game();
      update_scorebar(scoreY);
    } else {
      if (ballX<(bottomItemX+(bottomItemWidth/2)-1)) {
        itemDirX--;
      } else if (ballX>(bottomItemX+(bottomItemWidth/2)+1)) {
        itemDirX++;
      }
      tone(speakerPin, NOTE_C4,200);
    }
  }
  dmd.writePixel(ballX,ballY,7);
  if (oldBallY == gameCentreY || oldBallY == (gameCentreY-1)) {
    if ((oldBallY+oldBallX)&1) {
      dmd.writePixel(oldBallX,oldBallY,WHITE);
    } else {
      dmd.writePixel(oldBallX,oldBallY,BLACK);
    }
  }  else if ((ballX == gameMinX) || (ballX == gameMaxX)) {
    dmd.writePixel(oldBallX,oldBallY,WHITE);
  } else {
    dmd.writePixel(oldBallX,oldBallY,BLACK);
  }
 
}

void pong_display_score_update()
{
  tone(speakerPin,NOTE_C6,200);
  write_number(aiPoints,10,3,7);
  write_number(points,10,19,7);
  nunchuck_wait();
  int zbutton=0;
  while (!zbutton) {
    delay(50);
    nunchuck_get_data();
    if (nunchuck_cbutton()) {
      gameType=NONE;
      zbutton=true;
    } else {
      zbutton=nunchuck_zbutton();
    }
  }
  delay(200);
}
